"""
Determines whether this device is cpu or gpu.
"""

import torch  # pylint: disable=import-outside-toplevel


def get_computing_device() -> str:
    """Get the computing device."""
    try:
        if torch.cuda.is_available():
            return "cuda"
        return "cpu"
    except ImportError:
        return "cpu"


def _sanitize_filepath(folder_name: str) -> str:
    """Sanitize a folder name."""
    return "".join([c for c in folder_name if c.isalnum() or c in ["-", "_"]])


def sanitize_path(path: str) -> str:
    """Sanitizes a path."""
    # Generated by ai.
    out = (
        str(_sanitize_filepath(path))  # type: ignore
        .replace(" ", "_")
        .replace("/", "_")
        .replace("\\", "_")
        .replace(".", "_")
        .replace("-", "_")
        .replace(",", "_")
        .replace(":", "_")
        .replace(";", "_")
        .replace("(", "_")
        .replace(")", "_")
        .replace("[", "_")
        .replace("]", "_")
        .replace("{", "_")
        .replace("}", "_")
        .replace("<", "_")
        .replace(">", "_")
        .replace("?", "_")
        .replace("!", "_")
        .replace("@", "_")
        .replace("#", "_")
        .replace("$", "_")
        .replace("%", "_")
        .replace("^", "_")
        .replace("&", "_")
        .replace("*", "_")
        .replace("+", "_")
        .replace("=", "_")
        .replace("|", "_")
        .replace("~", "_")
        .replace("`", "_")
        .replace("'", "_")
        .replace('"', "_")
        .replace(" ", "_")
        .replace("\t", "_")
        .replace("\n", "_")
        .replace("\r", "_")
    )
    while len(out) > 4 and out[-1] == "_":
        out = out[:-1]
    while len(out) > 4 and "__" in out:
        out = out.replace("__", "_")
    return out


def chop_double_extension(path_name):
    """takes in a path like out.mp3.txt and returns out.mp3"""
    # Split the path name on "."
    parts = path_name.split(".")
    # If there are fewer than two parts, return the original path name
    if len(parts) < 2:
        return path_name
    # Otherwise, return the second-to-last part followed by the last part
    return parts[-2] + "." + parts[-1]



"""
    Fetches audio and handles transcoding it for usage in Mozilla's Deepspeech.
"""


import sys
import tempfile
import subprocess
import os
import static_ffmpeg  # type: ignore

_PROCESS_TIMEOUT = 4 * 60 * 60


def _ytdlp_download(url: str, outdir: str) -> str:
    """Downloads a file using ytdlp."""
    os.makedirs(outdir, exist_ok=True)
    # remove all files in the directory
    for file in os.listdir(outdir):
        os.remove(os.path.join(outdir, file))
    cmd = f'yt-dlp --no-check-certificate -f bestaudio {url}'
    subprocess.run(cmd, shell=True, cwd=outdir, check=True, timeout=_PROCESS_TIMEOUT)
    new_files = os.listdir(outdir)
    assert len(new_files) == 1, f"Expected 1 file, got {new_files}"
    print("New files: ", new_files)
    downloaded_file = os.path.join(outdir, new_files[0])
    print(f"Downloaded file: {downloaded_file}")
    assert os.path.exists(downloaded_file), f"The expected file {downloaded_file} doesn't exist"
    return downloaded_file


def _convert_to_mp3(inpath: str, outpath: str) -> None:
    """Converts a file to mp3."""
    cmd = f'static_ffmpeg -y -i "{inpath}" -acodec libmp3lame "{outpath}"'
    sys.stderr.write(f"Running:\n  {cmd}\n")
    subprocess.run(
        cmd, shell=True, check=True, capture_output=True, timeout=_PROCESS_TIMEOUT
    )
    assert os.path.exists(outpath), f"The expected file {outpath} doesn't exist"


def fetch_audio(url_or_file: str, out_mp3: str) -> None:
    """Fetches from the internet or from a local file and outputs a wav file."""
    assert out_mp3.endswith(".mp3")
    static_ffmpeg.add_paths()
    if url_or_file.startswith("http") or url_or_file.startswith("ftp"):
        with tempfile.TemporaryDirectory() as tmpdir:
            print(f"Using temporary directory {tmpdir}")
            downloaded_file = _ytdlp_download(url_or_file, os.path.abspath(tmpdir))
            print("Downloaded file: ", downloaded_file)
            _convert_to_mp3(downloaded_file, out_mp3)
        sys.stderr.write("Downloading complete.\n")
        assert os.path.exists(out_mp3), f"The expected file {out_mp3} doesn't exist"
    else:
        assert os.path.isfile(url_or_file)
        # static_ffmpeg -i audio.wav -acodec libmp3lame audio.mp3
        cmd = f'static_ffmpeg -i "{url_or_file}" -acodec libmp3lame "{out_mp3}"'
        sys.stderr.write(f"Running:\n  {cmd}\n")
        subprocess.run(
            cmd, shell=True, check=True, capture_output=True, timeout=_PROCESS_TIMEOUT
        )
        assert os.path.exists(out_mp3), f"The expected file {out_mp3} doesn't exist"


def unit_test() -> None:
    """Runs the program."""
    url = 'https://www.youtube.com/watch?v=8Wg8f2g_GQY'
    fetch_audio(url, 'out.mp3')


if __name__ == '__main__':
    unit_test()



"""
    Api for using transcribe_anything from python. Allows bulk processing.
"""

# pylint: disable=too-many-arguments,broad-except,too-many-locals

import os
import stat
import sys
import time
import subprocess

from transcribe_anything.audio import fetch_audio
from transcribe_anything.util import get_computing_device, sanitize_path, chop_double_extension

PERMS = (
    stat.S_IRUSR
    | stat.S_IRGRP
    | stat.S_IROTH
    | stat.S_IWOTH
    | stat.S_IWUSR
    | stat.S_IWGRP
)


def transcribe(
    url_or_file: str,
    output_dir: str | None = None,
    model: str | None = None,
    task: str | None = None,
    language: str | None = None
) -> str:
    """
    Runs the program.
    """
    basename = os.path.basename(url_or_file)
    if output_dir is None:
        output_dir = sanitize_path(basename)
    os.makedirs(output_dir, exist_ok=True)
    tmp_mp3 = os.path.join(output_dir, "out.mp3")
    fetch_audio(url_or_file, tmp_mp3)
    assert os.path.exists(tmp_mp3), f"Path {tmp_mp3} doesn't exist."
    device = get_computing_device()
    if device == "cuda":
        print("Using GPU")
    elif device == "cpu":
        print("WARNING: Using CPU, this will be at least 10x slower.")
    else:
        raise ValueError(f"Unknown device {device}")
    print(f"Using device {device}")
    model_str = f" --model {model}" if model else ""
    output_dir_str = f' --output_dir "{output_dir}"' if output_dir else ""
    task_str = f" --task {task}" if task else ""
    language_str = f" --language {language}" if language else ""
    cmd_list = [
        "whisper",
        tmp_mp3,
        "--device",
        device,
        model_str,
        output_dir_str,
        task_str,
        language_str,
    ]
    cmd = " ".join(cmd_list)
    sys.stderr.write(f"Running:\n  {cmd}\n")
    proc = subprocess.Popen(cmd, shell=True)  # pylint: disable=consider-using-with
    while True:
        rtn = proc.poll()
        if rtn is None:
            time.sleep(0.25)
            continue
        if rtn != 0:
            msg = f"Failed to execute {cmd}\n "
            raise OSError(msg)
        break
    files = [os.path.join(output_dir, name) for name in os.listdir(output_dir)]
    for file in files:
        # Change the filename to remove the double extension
        file_name = os.path.basename(file)
        base_path = os.path.dirname(file)
        new_file = os.path.join(base_path, chop_double_extension(file_name))
        if file != new_file:
            os.rename(file, new_file)
    return output_dir


if __name__ == "__main__":
    transcribe(url_or_file="https://www.youtube.com/watch?v=-4EDhdAHrOg")



"""
    Entry point for running the transcribe-anything prgram.
"""

import argparse
import sys

from transcribe_anything.api import transcribe
from transcribe_anything.util import get_computing_device
from transcribe_anything.parse_whisper_options import parse_whisper_options


def main() -> None:
    """Main entry point for the command line tool."""
    whisper_options = parse_whisper_options()
    device = get_computing_device()
    help_str = f'transcribe_anything is using a "{device}" device'
    parser = argparse.ArgumentParser(description=help_str)
    parser.add_argument(
        "url_or_file",
        help="Provide file path or url (includes youtube/facebook/twitter/etc)",
    )
    parser.add_argument(
        "--output_dir",
        help="Provide output directory name,d efaults to the filename of the file.",
        default=None,
    )
    parser.add_argument(
        "--model",
        help="name of the Whisper model to us",
        default="small",
        choices=whisper_options["model"],
    )
    parser.add_argument(
        "--task",
        help="whether to perform transcription or translation",
        default="transcribe",
        choices=whisper_options["task"],
    )
    parser.add_argument(
        "--language",
        help="language to the target audio is in, default None will auto-detect",
        default=None,
        choices=[None] + whisper_options["language"]
    )
    args = parser.parse_args()
    transcribe(
        url_or_file=args.url_or_file,
        output_dir=args.output_dir,
        model=args.model,
        task=args.task,
        language=args.language,
    )


if __name__ == "__main__":
    main()
    sys.exit(0)
